import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import Graph, { MultiGraph } from 'graphology';
import { SerializedGraph } from 'graphology-types';
import { SchemaFromBackend } from 'libs/shared/models/src';
import type { RootState } from './store';

/**************************************************************** */

// Define the initial state using that type
export const initialState = {
  graphologySerialized: new MultiGraph().export(),
  layoutName: 'Cytoscape_fcose',
};

export const schemaSlice = createSlice({
  name: 'schema',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    setSchema: (state, action: PayloadAction<SerializedGraph>) => {
      console.log('setSchema', action);
      state.graphologySerialized = action.payload;
    },

    setSchemaLayout: (state, action: PayloadAction<string>) => {
      console.log('setSchemaLayout', action);

      state.layoutName = action.payload;
    },

    readInSchemaFromBackend: (
      state,
      action: PayloadAction<SchemaFromBackend>
    ) => {
      const { nodes, edges } = action.payload;
      // Instantiate a directed graph that allows self loops and parallel edges
      const schema = new MultiGraph({ allowSelfLoops: true });
      console.log('Updating schema');
      // The graph schema needs a node for each node AND edge. These need then be connected

      // nodes.forEach((node) => {
      //   schema.addNode(node.name, {
      //     name: node.name,
      //     attributes: node.attributes,
      //     x: 0,
      //     y: 0,
      //   });
      // });

      // // The name of the edge will be name + from + to, since edge names are not unique
      // edges.forEach((edge) => {
      //   const edgeID = edge.name + edge.from + edge.to;

      //   // This node is the actual edge
      //   schema.addNode(edgeID, {
      //     name: edge.name,
      //     attributes: edge.attributes,
      //     from: edge.from,
      //     to: edge.to,
      //     collection: edge.collection,
      //     x: 0,
      //     y: 0,
      //   });

      //   // These lines are simply for keeping the schema together
      //   schema.addDirectedEdgeWithKey(edgeID + 'f', edge.from, edgeID);
      //   schema.addDirectedEdgeWithKey(edgeID + 't', edgeID, edge.to);
      // });

      state.graphologySerialized = schema.export();
    },
  },
});

export const { readInSchemaFromBackend, setSchema } = schemaSlice.actions;

/**
 * Select the schema and convert it to a graphology object
 * */
export const selectSchema = (state: RootState) => {
  // console.log(state);
  return MultiGraph.from(state.schema.graphologySerialized);
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
export const selectSchemaLayout = (state: RootState) => state.schema.layoutName;

export default schemaSlice.reducer;
